@@ -34,6 +34,7 @@ pub mod ledger_cleanup_service;
 pub mod optimistic_confirmation_verifier;
 pub mod outstanding_requests;
 pub mod packet_hasher;
+pub mod packet_threshold;
 pub mod progress_map;
 pub mod qos_service;
 pub mod repair_generic_traversal;
@@ -54,7 +55,9 @@ pub mod sigverify;
 pub mod sigverify_shreds;
 pub mod sigverify_stage;
 pub mod snapshot_packager_service;
+pub mod stats_reporter_service;
 pub mod system_monitor_service;
+mod tower1_7_14;
 pub mod tower_storage;
 pub mod tpu;
 pub mod tree_diff;
