@@ -8,15 +8,16 @@
 use {
     crate::sigverify,
     core::time::Duration,
-    crossbeam_channel::{SendError, Sender as CrossbeamSender},
+    crossbeam_channel::{Receiver, RecvTimeoutError, SendError, Sender},
     itertools::Itertools,
     solana_measure::measure::Measure,
-    solana_perf::packet::PacketBatch,
-    solana_perf::sigverify::{count_valid_packets, shrink_batches, Deduper},
+    solana_perf::{
+        packet::PacketBatch,
+        sigverify::{count_valid_packets, shrink_batches, Deduper},
+    },
     solana_sdk::timing,
     solana_streamer::streamer::{self, PacketBatchReceiver, StreamerError},
     std::{
-        sync::mpsc::{Receiver, RecvTimeoutError},
         thread::{self, Builder, JoinHandle},
         time::Instant,
     },
@@ -41,7 +42,7 @@ pub struct SigVerifyStage {
 }
 
 pub trait SigVerifier {
-    fn verify_batches(&self, batches: Vec<PacketBatch>) -> Vec<PacketBatch>;
+    fn verify_batches(&self, batches: Vec<PacketBatch>, valid_packets: usize) -> Vec<PacketBatch>;
 }
 
 #[derive(Default, Clone)]
@@ -178,7 +179,11 @@ impl SigVerifierStats {
 }
 
 impl SigVerifier for DisabledSigVerifier {
-    fn verify_batches(&self, mut batches: Vec<PacketBatch>) -> Vec<PacketBatch> {
+    fn verify_batches(
+        &self,
+        mut batches: Vec<PacketBatch>,
+        _valid_packets: usize,
+    ) -> Vec<PacketBatch> {
         sigverify::ed25519_verify_disabled(&mut batches);
         batches
     }
@@ -188,7 +193,7 @@ impl SigVerifyStage {
     #[allow(clippy::new_ret_no_self)]
     pub fn new<T: SigVerifier + 'static + Send + Clone>(
         packet_receiver: Receiver<PacketBatch>,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        verified_sender: Sender<Vec<PacketBatch>>,
         verifier: T,
     ) -> Self {
         let thread_hdl = Self::verifier_services(packet_receiver, verified_sender, verifier);
@@ -201,6 +206,7 @@ impl SigVerifyStage {
             .iter_mut()
             .rev()
             .flat_map(|batch| batch.packets.iter_mut().rev())
+            .filter(|packet| !packet.meta.discard())
             .map(|packet| (packet.meta.addr, packet))
             .into_group_map();
         // Allocate max_packets evenly across addresses.
@@ -222,7 +228,7 @@ impl SigVerifyStage {
     fn verifier<T: SigVerifier>(
         deduper: &Deduper,
         recvr: &PacketBatchReceiver,
-        sendr: &CrossbeamSender<Vec<PacketBatch>>,
+        sendr: &Sender<Vec<PacketBatch>>,
         verifier: &T,
         stats: &mut SigVerifierStats,
     ) -> Result<()> {
@@ -241,14 +247,16 @@ impl SigVerifyStage {
         let num_unique = num_packets.saturating_sub(dedup_fail);
 
         let mut discard_time = Measure::start("sigverify_discard_time");
+        let mut num_valid_packets = num_unique;
         if num_unique > MAX_SIGVERIFY_BATCH {
-            Self::discard_excess_packets(&mut batches, MAX_SIGVERIFY_BATCH)
-        };
+            Self::discard_excess_packets(&mut batches, MAX_SIGVERIFY_BATCH);
+            num_valid_packets = MAX_SIGVERIFY_BATCH;
+        }
         let excess_fail = num_unique.saturating_sub(MAX_SIGVERIFY_BATCH);
         discard_time.stop();
 
         let mut verify_batch_time = Measure::start("sigverify_batch_time");
-        let mut batches = verifier.verify_batches(batches);
+        let mut batches = verifier.verify_batches(batches, num_valid_packets);
         verify_batch_time.stop();
 
         let mut shrink_time = Measure::start("sigverify_shrink_time");
@@ -305,7 +313,7 @@ impl SigVerifyStage {
 
     fn verifier_service<T: SigVerifier + 'static + Send + Clone>(
         packet_receiver: PacketBatchReceiver,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        verified_sender: Sender<Vec<PacketBatch>>,
         verifier: &T,
     ) -> JoinHandle<()> {
         let verifier = verifier.clone();
@@ -351,7 +359,7 @@ impl SigVerifyStage {
 
     fn verifier_services<T: SigVerifier + 'static + Send + Clone>(
         packet_receiver: PacketBatchReceiver,
-        verified_sender: CrossbeamSender<Vec<PacketBatch>>,
+        verified_sender: Sender<Vec<PacketBatch>>,
         verifier: T,
     ) -> JoinHandle<()> {
         Self::verifier_service(packet_receiver, verified_sender, &verifier)
@@ -364,13 +372,15 @@ impl SigVerifyStage {
 
 #[cfg(test)]
 mod tests {
-    use crate::sigverify::TransactionSigVerifier;
-    use crate::sigverify_stage::timing::duration_as_ms;
-    use crossbeam_channel::unbounded;
-    use solana_perf::packet::to_packet_batches;
-    use solana_perf::test_tx::test_tx;
-    use std::sync::mpsc::channel;
-    use {super::*, solana_perf::packet::Packet};
+    use {
+        super::*,
+        crate::{sigverify::TransactionSigVerifier, sigverify_stage::timing::duration_as_ms},
+        crossbeam_channel::unbounded,
+        solana_perf::{
+            packet::{to_packet_batches, Packet},
+            test_tx::test_tx,
+        },
+    };
 
     fn count_non_discard(packet_batches: &[PacketBatch]) -> usize {
         packet_batches
@@ -391,12 +401,15 @@ mod tests {
         let mut batch = PacketBatch::default();
         batch.packets.resize(10, Packet::default());
         batch.packets[3].meta.addr = std::net::IpAddr::from([1u16; 8]);
+        batch.packets[3].meta.set_discard(true);
+        batch.packets[4].meta.addr = std::net::IpAddr::from([2u16; 8]);
         let mut batches = vec![batch];
         let max = 3;
         SigVerifyStage::discard_excess_packets(&mut batches, max);
         assert_eq!(count_non_discard(&batches), max);
         assert!(!batches[0].packets[0].meta.discard());
-        assert!(!batches[0].packets[3].meta.discard());
+        assert!(batches[0].packets[3].meta.discard());
+        assert!(!batches[0].packets[4].meta.discard());
     }
     fn gen_batches(use_same_tx: bool) -> Vec<PacketBatch> {
         let len = 4096;
@@ -414,7 +427,7 @@ mod tests {
     fn test_sigverify_stage() {
         solana_logger::setup();
         trace!("start");
-        let (packet_s, packet_r) = channel();
+        let (packet_s, packet_r) = unbounded();
         let (verified_s, verified_r) = unbounded();
         let verifier = TransactionSigVerifier::default();
         let stage = SigVerifyStage::new(packet_r, verified_s, verifier);
