@@ -13,7 +13,7 @@ use {
         sigverify::TransactionSigVerifier,
         sigverify_stage::SigVerifyStage,
     },
-    crossbeam_channel::unbounded,
+    crossbeam_channel::{unbounded, Receiver},
     solana_gossip::cluster_info::ClusterInfo,
     solana_ledger::{blockstore::Blockstore, blockstore_processor::TransactionStatusSender},
     solana_poh::poh_recorder::{PohRecorder, WorkingBankEntry},
@@ -29,17 +29,21 @@ use {
     solana_sdk::signature::Keypair,
     std::{
         net::UdpSocket,
-        sync::{
-            atomic::AtomicBool,
-            mpsc::{channel, Receiver},
-            Arc, Mutex, RwLock,
-        },
+        sync::{atomic::AtomicBool, Arc, Mutex, RwLock},
         thread,
     },
 };
 
 pub const DEFAULT_TPU_COALESCE_MS: u64 = 5;
 
+pub struct TpuSockets {
+    pub transactions: Vec<UdpSocket>,
+    pub transaction_forwards: Vec<UdpSocket>,
+    pub vote: Vec<UdpSocket>,
+    pub broadcast: Vec<UdpSocket>,
+    pub transactions_quic: UdpSocket,
+}
+
 pub struct Tpu {
     fetch_stage: FetchStage,
     sigverify_stage: SigVerifyStage,
@@ -57,11 +61,7 @@ impl Tpu {
         poh_recorder: &Arc<Mutex<PohRecorder>>,
         entry_receiver: Receiver<WorkingBankEntry>,
         retransmit_slots_receiver: RetransmitSlotsReceiver,
-        transactions_sockets: Vec<UdpSocket>,
-        tpu_forwards_sockets: Vec<UdpSocket>,
-        tpu_vote_sockets: Vec<UdpSocket>,
-        broadcast_sockets: Vec<UdpSocket>,
-        transactions_quic_socket: UdpSocket,
+        sockets: TpuSockets,
         subscriptions: &Arc<RpcSubscriptions>,
         transaction_status_sender: Option<TransactionStatusSender>,
         blockstore: &Arc<Blockstore>,
@@ -80,8 +80,16 @@ impl Tpu {
         cost_model: &Arc<RwLock<CostModel>>,
         keypair: &Keypair,
     ) -> Self {
-        let (packet_sender, packet_receiver) = channel();
-        let (vote_packet_sender, vote_packet_receiver) = channel();
+        let TpuSockets {
+            transactions: transactions_sockets,
+            transaction_forwards: tpu_forwards_sockets,
+            vote: tpu_vote_sockets,
+            broadcast: broadcast_sockets,
+            transactions_quic: transactions_quic_sockets,
+        } = sockets;
+
+        let (packet_sender, packet_receiver) = unbounded();
+        let (vote_packet_sender, vote_packet_receiver) = unbounded();
         let fetch_stage = FetchStage::new_with_sender(
             transactions_sockets,
             tpu_forwards_sockets,
@@ -95,7 +103,7 @@ impl Tpu {
         let (verified_sender, verified_receiver) = unbounded();
 
         let tpu_quic_t = solana_streamer::quic::spawn_server(
-            transactions_quic_socket,
+            transactions_quic_sockets,
             keypair,
             cluster_info.my_contact_info().tpu.ip(),
             packet_sender,
